from zope.interface import implements
from interfaces import ITransport, TransportError
import ftputil
from ftputil import ftp_error
import os

class FtpTransport(object):
    implements(ITransport)
    
    def __init__(self, hostname, username, password,
                 target_directory):
        self.host = self.ftpHostFactory(hostname, username, password)
        self.target_directory = target_directory
        self.upload_directory = os.path.join(target_directory, 'XXX_upload')
        self.purge_directory = os.path.join(target_directory, 'XXX_purge')

    def close(self):
        self.host.close()

    def ftpHostFactory(self, *args, **kw):
        return ftputil.FTPHost(*args, **kw)

    def isIllegalName(self, name):
        # XXX simplistic check, extend this to only allow names that
        # can be FTP-ed..
        return name.startswith(' ') or name.startswith('.')
    
    def upload(self, source_directory):
        """Uploads local files in source_directory into ftp server.

        Also purges any file in the target_directory if
        that file doesn't exist in the list of file we just uploaded.
        """
        # construct start point of path relevant to uploads
        # have to add 1 to eliminate trailing slash
        source_directory = os.path.normpath(source_directory)
        self._source_start = len(source_directory) + 1

        try:
            try:
                self._createUploadDirectory()
                self._uploadContents(source_directory)
                self._createPurgeDirectory()
                self._purgeOriginalContents()
                self._commitUploadedContents()
            finally:
                # we always want to do this, no matter whether something
                # went wrong
                self._cleanupTemporaryDirectories()
        except ftp_error.FTPError, e:
            raise TransportError(e.strerror)
    
    def _createUploadDirectory(self):
        if not self.host.path.isdir(self.upload_directory):
            self.host.mkdir(self.upload_directory)

    def _uploadContents(self, source_directory):
        for dirpath, dirnames, filenames in os.walk(source_directory):
            # filtering out directory names from dirnames will cause
            # os.walk to skip that directory
            self._filterNames(dirnames)
            self._createDirectories(dirpath, dirnames)
            self._filterNames(filenames)
            self._uploadFiles(dirpath, filenames)

    def _filterNames(self, names):
        """Modifies names in place, removing any names that are illegal.
        """
        for i, name in enumerate(names):
            if self.isIllegalName(name):
                del names[i]
        
    def _createDirectories(self, dirpath, dirnames):
        """Create any remote subdirectories in dirpath that are local.
        """
        for dirname in dirnames:
            full_dirname = os.path.join(dirpath, dirname)
            dir_to_create = self._getTargetPath(full_dirname)
            if not self.host.path.isdir(dir_to_create):
                self.host.mkdir(dir_to_create)

    def _uploadFiles(self, dirpath, filenames):
        """Upload all files that need uploading into directory.
        """
        for filename in filenames:
            full_filename = os.path.join(dirpath, filename)
            self.host.upload(full_filename, self._getTargetPath(full_filename),
                             mode='b')

    def _createPurgeDirectory(self):
        if not self.host.path.isdir(self.purge_directory):
            self.host.mkdir(self.purge_directory)

    def _purgeOriginalContents(self):
        for filename in self.host.listdir(self.target_directory):
            orig_path = os.path.join(self.target_directory, filename)
            if orig_path in [self.upload_directory, self.purge_directory]:
                continue

            try:
                self.host.rename(orig_path,
                                 os.path.join(self.purge_directory, filename))
            except ftp_error.FTPError, e:
                # just skip this resource..
                pass # print "Could not purge:", orig_path
                
    def _commitUploadedContents(self):
        for filename in self.host.listdir(self.upload_directory):
            self.host.rename(os.path.join(self.upload_directory, filename),
                             os.path.join(self.target_directory, filename))

    def _cleanupTemporaryDirectories(self):
        self.host.rmtree(self.purge_directory, ignore_errors=True)
        self.host.rmtree(self.upload_directory, ignore_errors=True)

    def _getTargetPath(self, source_path):
        source_path = os.path.normpath(source_path)
        return os.path.join(self.upload_directory,
                            source_path[self._source_start:])

def test():
    transport = FTPTransport(
        'localhost', 'tempuser', 'testpw',
        'dump')
    transport.upload('/home/faassen/working/static/localhost:8080/silva')

if __name__ == '__main__':
    test()

        
