import unittest
import os, shutil, tempfile
from Products.SilvaStatic import ftptransport

class MockFtpHost(object):
    def __init__(self, directory):
        self.path = MockFtpPath(self)
        self.directory = directory
        
    def rename(self, from_path, to_path):
        os.rename(self._ftpPath(from_path),
                  self._ftpPath(to_path))

    def remove(self, path):
        os.remove(self._ftpPath(path))

    def rmtree(self, path, ignore_errors=False):
        shutil.rmtree(self._ftpPath(path), ignore_errors)

    def mkdir(self, path):
        os.mkdir(self._ftpPath(path))

    def upload(self, source_path, target_path, mode):
        shutil.copy(source_path, self._ftpPath(target_path))

    def close(self):
        shutil.rmtree(self.directory, ignore_errors=True)

    def listdir(self, path):
        return os.listdir(self._ftpPath(path))

    def _ftpPath(self, path):
        return os.path.join(self.directory, path)

    def isIdentical(self, source, target):
        return self._isIdenticalHelper(source, self._ftpPath(target))
    
    def _isIdenticalHelper(self, source, target):
        """Verify whether our directory contains identical information
        compared to path.
        """
        source_entries = os.listdir(source)
        target_entries = os.listdir(target)
        source_entries.sort()
        target_entries.sort()
        if source_entries != target_entries:
            return False
        for source_entry in source_entries:
            source_entry_path = os.path.join(source, source_entry)
            target_entry_path = os.path.join(target, source_entry)
            if (os.path.isfile(source_entry_path) and
                os.path.isfile(target_entry_path)):
                source_f = open(source_entry_path, 'rb')
                source_data = source_f.read()
                source_f.close()
                target_f = open(target_entry_path, 'rb')
                target_data = target_f.read()
                target_f.close()
                if source_data != target_data:
                    return False
            elif (os.path.isdir(source_entry_path) and
                  os.path.isdir(target_entry_path)):
                self._isIdenticalHelper(source_entry_path, target_entry_path)
            else:
                return False
        return True
        
class MockFtpPath(object):
    def __init__(self, host):
        self.host = host

    def exists(self, path):
        return os.path.exists(self.host._ftpPath(path))

    def isdir(self, path):
        return os.path.isdir(self.host._ftpPath(path))
        
    def isfile(self, path):
        return os.path.isfile(self.host._ftpPath(path))

    
class MockFtpHostTestCase(unittest.TestCase):
    def test_upload(self):
        temp_dir = tempfile.mkdtemp()
        host = MockFtpHost(temp_dir)
        host.mkdir('sub')
        host.upload(testdata('test.txt'), 'sub/test.txt', 'b')
        self.assert_(
            host.path.isfile('sub/test.txt'))
        self.assert_(
            host.path.isdir('sub'))
        self.assert_(
            not host.path.isdir('sub/test.txt'))
        self.assert_(
            not host.path.isfile('sub'))
        self.assert_(
            not host.path.isdir('foo'))
        self.assert_(
            not host.path.isfile('foo'))
        self.assert_(
            host.path.exists('sub'))
        self.assert_(
            host.path.exists('sub/test.txt'))
        self.assert_(
            not host.path.exists('foo'))
        self.assertEquals(
            ['test.txt'], host.listdir('sub'))
        host.remove('sub/test.txt')
        self.assert_(
            not host.path.exists('sub/test.txt'))
        self.assertEquals(
            [], host.listdir('sub'))
        host.close()
        
class MockFtpTransport(ftptransport.FtpTransport):
    def __init__(self, source_directory, target_directory):
        self.source_directory = source_directory
        super(MockFtpTransport, self).__init__('dummy', 'dummy', 'dummy',
                                               target_directory)
        
    def ftpHostFactory(self, *args, **kw):
        return MockFtpHost(self.source_directory)

class SynchTestCase(unittest.TestCase):
    def test_synch_d(self):
        ftp_dir = tempfile.mkdtemp()
        os.mkdir(os.path.join(ftp_dir, 'sub'))
        
        transport = MockFtpTransport(ftp_dir, 'sub')

        for entry in ['d1', 'd2', 'd3', 'd4', 'd5', 'd6', 'd7']:
            path = testdata(entry)
            transport.upload(path)
            self.assert_(transport.host.isIdentical(path, 'sub'))

        transport.close()

    
def testdata(name):
    dir, dummy = os.path.split(__file__)
    return os.path.join(dir, 'data', name)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(MockFtpHostTestCase))
    suite.addTest(unittest.makeSuite(SynchTestCase))
    return suite
